package cn.com.qmth.stmms.biz.report.service.impl;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.dao.ExamStudentDao;
import cn.com.qmth.stmms.biz.exam.dao.ExamSubjectDao;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.report.dao.ReportSubjectQuestionDao;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectQuestion;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectQuestionService;
import cn.com.qmth.stmms.biz.utils.DoubleUtil;
import cn.com.qmth.stmms.biz.utils.ScoreItem;

/**
 * 客、主观题分析 service 实现
 *
 * @author xuwenjin
 * @Date 2019-04-17
 */
@Service
public class ReportSubjectQuestionServiceImpl extends BaseQueryService<ReportSubjectQuestion> implements ReportSubjectQuestionService {

    @Autowired
    private ReportSubjectQuestionDao reportSubjectQuestionDao;

    @Autowired
    private ExamStudentDao examStudentDao;

    @Autowired
    private ExamQuestionService questionService;
    
    @Autowired
    private ExamSubjectDao examSubjectDao;
    
    @Override
    public ReportSubjectQuestion save(ReportSubjectQuestion reportSubjectQuestion) {
        return reportSubjectQuestionDao.save(reportSubjectQuestion);
    }

    @Transactional
    @Override
    public void deleteByExamId(Integer examId) {
        reportSubjectQuestionDao.deleteByExamId(examId);

    }

    @Override
    public List<ReportSubjectQuestion> findByQuery(final ReportSubjectQuery query) {
        query.setSort(new Sort(Direction.ASC, "paperType","mainNumber", "subNumber"));
        Page<ReportSubjectQuestion> result = reportSubjectQuestionDao.findAll(new Specification<ReportSubjectQuestion>() {
            @Override
            public Predicate toPredicate(Root<ReportSubjectQuestion> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();
                if (query.getExamId() != null) {
                    predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                }
                if (StringUtils.isNotBlank(query.getPaperType())) {
                    predicates.add(cb.equal(root.get("paperType"), query.getPaperType()));
                }
                if (StringUtils.isNotBlank(query.getSubjectCode())) {
                    predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                }
                predicates.add(cb.equal(root.get("objective"), query.getObjective()));
                return predicates.isEmpty() ? cb.conjunction()
                        : cb.and(predicates.toArray(new Predicate[predicates.size()]));
            }
        },query);
        return result.getContent();
    }

    @Override
    public void saveReportSubjectQuestionData(Integer examId, String subjectCode, Boolean objective) {
        Map<String,List<ExamQuestion>> map = new HashMap<String, List<ExamQuestion>>();
        ExamSubject subject = examSubjectDao.findByExamIdAndCode(examId, subjectCode);
        if(objective){
            List<ExamQuestion> examQuestionList = questionService.findByExamAndSubjectAndObjective(examId, subjectCode, objective);
            Set<String> paperTypeSet = new HashSet<String>();
            for (ExamQuestion examQuestion : examQuestionList) {
                paperTypeSet.add(examQuestion.getPaperType());
            }
            for (String paperType : paperTypeSet) {
                List<ExamQuestion> oList = questionService.findByExamAndSubjectAndObjectiveAndPaperType(examId, subjectCode, objective, paperType);
                map.put(subjectCode+"_"+paperType, oList);
            }
        }else{
            List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjective(examId, subjectCode, objective);
            map.put(subjectCode, sList);
        }

        List<ExamStudent> examStudentList = examStudentDao.findByExamIdAndSubjectCodeAndUploadTimeNotNullAndAbsent(examId, subjectCode, false);

        Integer studentCount = examStudentList.size();
        for(String key:map.keySet()){
            List<ExamQuestion> examQuestionList = map.get(key);
            for (int i = 0; i < examQuestionList.size(); i++) {
                ExamQuestion question = examQuestionList.get(i);
                if (question.getTotalScore().doubleValue() > 0) {
                    ReportSubjectQuestion reportSubjectQuestion = new ReportSubjectQuestion();
                    //得分人数
                    Integer scoreCount = 0;
                    //满分人数
                    Integer fullScoreCount = 0;
                    //考生题目得分汇总
                    Double sumScore = 0d;
                    //题目总分汇总
                    //Double questionSumScore = DoubleUtil.mul(question.getTotalScore(), studentCount);
                    //方差
                    Double variance = 0d;
                    
                    for (int j = 0; j < studentCount; j++) {
                        ExamStudent student = examStudentList.get(j);
                        if(question.getPaperType()!=null && !question.getPaperType().equalsIgnoreCase(student.getPaperType())){
                            continue;
                        }
                        List<ScoreItem> scoreItems = student.getScoreList(objective);
                        double score = scoreItems.get(i).getScore();
                        sumScore = DoubleUtil.sum(sumScore, score);
                        if (score > 0) {
                            scoreCount++;
                        }
                        if (score == question.getTotalScore().doubleValue()) {
                            fullScoreCount++;
                        }
                    }
                    Double avgScore = DoubleUtil.div(sumScore, examStudentList.size(), 2);
                    for (int j = 0; j < studentCount; j++) {
                        ExamStudent student = examStudentList.get(j);
                        if(question.getPaperType()!=null && !question.getPaperType().equalsIgnoreCase(student.getPaperType())){
                            continue;
                        }
                        List<ScoreItem> scoreItems = student.getScoreList(objective);
                        double score = scoreItems.get(i).getScore();
                        variance = DoubleUtil.sum(variance, DoubleUtil.mul(DoubleUtil.sub(score, avgScore), DoubleUtil.sub(score, avgScore)));
                    }
                    reportSubjectQuestion.setExamId(examId);
                    reportSubjectQuestion.setSubjectCode(subjectCode);
                    reportSubjectQuestion.setSubjectName(subject.getName());
                    reportSubjectQuestion.setObjective(objective);
                    reportSubjectQuestion.setQuestionName(question.getMainTitle());
                    reportSubjectQuestion.setMainNumber(question.getMainNumber());
                    reportSubjectQuestion.setSubNumber(question.getSubNumber());
                    reportSubjectQuestion.setTotalScore(question.getTotalScore());
                    reportSubjectQuestion.setAvgScore(avgScore);
                    reportSubjectQuestion.setScoreRate(DoubleUtil.div(avgScore, question.getTotalScore(), 2));
                    reportSubjectQuestion.setStdev(studentCount==0?0:DoubleUtil.format(Math.sqrt(variance / studentCount), "0.00"));
                    reportSubjectQuestion.setFullScoreRate(DoubleUtil.div(fullScoreCount, studentCount, 2));
                    reportSubjectQuestion.setPaperType(question.getPaperType());
                    reportSubjectQuestionDao.save(reportSubjectQuestion);
                }
            }
        }

    }


}
